using MailKit.Net.Pop3;
using Microsoft.Azure.WebJobs;
using Microsoft.Azure.WebJobs.Host;
using Microsoft.WindowsAzure.Storage;
using Microsoft.WindowsAzure.Storage.Blob;
using Microsoft.WindowsAzure.Storage.Table;
using System;
using System.Configuration;
using System.Net;

namespace Pop3Email
{
    public static class PopEmail
    {
        public class EmailMessage : TableEntity
        {
            public DateTime EmailDate { get; set; }
            public string EmailFrom { get; set; }
            public string EmailTo { get; set; }
            public string EmailSubject { get; set; }
        }

        // NOte: "0 */5 * * * *" will run once every 5 minutes. 
        // Change it to: "0 */1 * * * *" to run once a minute
        [FunctionName("PopEmail")]
        public static void Run([TimerTrigger("0 */5 * * * *")]TimerInfo myTimer, 
            [Table("pop3messages", Connection = "AzureWebJobsStorage")]CloudTable outputTable,
            TraceWriter log)
        {
            log.Info("C# HTTP trigger function processed a request.");

            string _cloudUploadedFilesContainerName = "pop3-messages";
            string _storageConnectionString = ConfigurationManager.AppSettings["AzureWebJobsStorage"];
            string _POP3Server = ConfigurationManager.AppSettings["POP3Server"];
            int _POP3Port = Convert.ToInt32(ConfigurationManager.AppSettings["POP3Port"]);
            bool _POP3UseSSL = Convert.ToBoolean(ConfigurationManager.AppSettings["POP3UseSSL"]);
            string _POP3Username = ConfigurationManager.AppSettings["POP3Username"];
            string _POP3Password = ConfigurationManager.AppSettings["POP3Password"];

            using (var client = new Pop3Client())
            {
                // For demo-purposes, accept all SSL certificates (in case the server supports STARTTLS)
                client.ServerCertificateValidationCallback = (s, c, h, e) => true;

                client.Connect(_POP3Server, _POP3Port, _POP3UseSSL);

                // Note: since we don't have an OAuth2 token, disable
                // the XOAUTH2 authentication mechanism.
                client.AuthenticationMechanisms.Remove("XOAUTH2");

                client.Authenticate(_POP3Username, _POP3Password);

                for (int i = 0; i < client.Count; i++)
                {
                    string strMessageGUID = Guid.NewGuid().ToString();
                    var message = client.GetMessage(i);

                    var NewItem = new EmailMessage();
                    NewItem.EmailDate = message.Date.Date;
                    NewItem.EmailSubject = message.Subject;

                    log.Info($"Got Message: {message.Subject}");

                    string strFromAddresses = "";
                    foreach (var item in message.From.Mailboxes)
                    {
                        strFromAddresses += item.Address + ";";
                    }
                    NewItem.EmailFrom = strFromAddresses;

                    // Get all Addresses
                    string strToAddresses = "";
                    foreach (var item in message.To.Mailboxes)
                    {
                        strToAddresses += item.Address + ";";
                    }
                    NewItem.EmailTo = strToAddresses;

                    NewItem.PartitionKey = "pop3";
                    NewItem.RowKey = strMessageGUID;
                    NewItem.ETag = "*";

                    try
                    {
                        var operation = TableOperation.Insert(NewItem);
                        outputTable.ExecuteAsync(operation);
                    }
                    catch (Exception ex)
                    {
                        log.Info(ex.Message);
                    }

                    // Save the Message as a Blob because it is too big to fit in a Table
                    var EmailContent = ((MimeKit.TextPart)message.Body).Text;

                    var _cloudStorageAccount = CloudStorageAccount.Parse(_storageConnectionString);
                    var _cloudBlobClient = _cloudStorageAccount.CreateCloudBlobClient();
                    var _cloudFilesBlobContainer =
                        _cloudBlobClient.GetContainerReference(_cloudUploadedFilesContainerName);

                    CloudBlockBlob targetBlockBlob = 
                        _cloudFilesBlobContainer.GetBlockBlobReference(strMessageGUID);

                    log.Info($"UploadFromStream: {strMessageGUID}");
                    targetBlockBlob.UploadTextAsync(EmailContent);

                    // [OPTIONAL] Mark the message for deletion
                    // client.DeleteMessage(i);
                }

                log.Info("client.Disconnect");
                client.Disconnect(true);
            }
        }
    }
}
